// CONVEKS - Copyright (C) 2018 E. Kuci, C. Geuzaine and P. Duysinx, University of Liege
//
// See the LICENSE.txt file for license information. Please report all
// bugs and problems to the public mailing list <conveks@onelab.info>.

#ifndef _CONVEKS_H_
#define _CONVEKS_H_

// This file defines the CONVEKS C++ API (v1.1).
//
// Do not edit it directly: it is automatically generated by `api/gen.py'.
//
// By design, the CONVEKS C++ API is purely functional, and only uses elementary
// types from the standard library. See `demos/api' for examples.

#if defined(_MSC_VER) && !defined(_USE_MATH_DEFINES)
#define _USE_MATH_DEFINES
#endif

#include <cmath>
#include <vector>
#include <string>
#include <utility>
#include <limits>

#define CONVEKS_API_VERSION "1.1"

#if defined(CONVEKS_DLL)
#if defined(CONVEKS_DLL_EXPORT)
#define CONVEKS_API __declspec(dllexport)
#else
#define CONVEKS_API __declspec(dllimport)
#endif
#else
#define CONVEKS_API
#endif

namespace conveks {

  // A geometrical entity in the CONVEKS API is represented by two integers: its
  // dimension (dim = 0, 1, 2 or 3) and its tag (its unique, strictly positive
  // identifier). When dealing with multiple geometrical entities of possibly
  // different dimensions, the entities are packed as a vector of (dim, tag)
  // integer pairs.
  typedef std::vector<std::pair<int, int> > vectorpair;

}

namespace conveks { // Top-level functions

  // Initialize Conveks.
  CONVEKS_API void initialize(const std::string & type = "none");

  // Solve the optimization problem.
  CONVEKS_API void run();

  // Add the objective function.
  CONVEKS_API void addObjective(const void * func);

  // Solve the optimization problem.
  CONVEKS_API void addConstraint();

  // Solve the optimization problem.
  CONVEKS_API void addGroupOfConstraints();

  // Solve the optimization problem.
  CONVEKS_API void addVariable();

  // Solve the optimization problem.
  CONVEKS_API void addGroupOfVariables();

  // Compute the objective function.
  CONVEKS_API void computeObjective();

  // Finalize Conveks.
  CONVEKS_API void finalize();

  namespace option { // Global option handling functions

    // Set a numerical option to `value'. `name'. Available categories and options
    // are listed in the Conveks reference manual.
    CONVEKS_API void setNumber(const std::string & name,
                               const double value);

    // Get the `value' of a numerical option.
    CONVEKS_API void getNumber(const std::string & name,
                               double & value);

    // Get all the options.
    CONVEKS_API void getList(std::vector<std::string> & names);

    // Print the numerical options.
    CONVEKS_API void show();

  } // namespace option

  namespace mma { // MMA optimizer

    // Initialize MMA.
    CONVEKS_API void initialize(const std::vector<double> & currentPoint,
                                const std::vector<double> & lowerBound,
                                const std::vector<double> & upperBound,
                                const bool restart = false);

    // Get current point.
    CONVEKS_API void getCurrentPoint(std::vector<double> & currentPoint);

    // Update current point (based on first order derivatives and possibly the
    // diagonal entries of the objective function hessian).
    CONVEKS_API void updateCurrentPoint(const double objective,
                                        const std::vector<double> & gradientOfObjective,
                                        const std::vector<double> & constraints = std::vector<double>(),
                                        const std::vector<double> & gradientOfConstraints = std::vector<double>(),
                                        const std::vector<double> & diagHessianOfObjective = std::vector<double>(),
                                        const double objectiveAtUpdatedPoint = std::numeric_limits<double>::quiet_NaN(),
                                        const std::vector<double> & constraintsAtUpdatedPoint = std::vector<double>());

    // Get the norm of KKT at current point.
    CONVEKS_API void getKKTNorm(double & norm,
                                const std::vector<double> & constraints,
                                const std::vector<double> & gradientOfObjective,
                                const std::vector<double> & gradientOfConstraints);

    // Check if the submodel is conservative and return 1 if it is the case,
    // otherwise return 0.
    CONVEKS_API void isConservativeSubProblem(int & conservative,
                                              const double objective,
                                              const std::vector<double> & constraints = std::vector<double>());

    // Get outer iteration.
    CONVEKS_API void getOuterIteration(int & iteration);

    // Get total number of nonlinear iterations.
    CONVEKS_API void countNonlinearIterations(int & iteration);

    // Get the infinite norm between two successive designs.
    CONVEKS_API void getDesignChange(double & change);

    // Set the outer move limits.
    CONVEKS_API void setMoveLimits(const std::vector<double> & lowerBound,
                                   const std::vector<double> & upperBound,
                                   const double move);

    // Finalize MMA.
    CONVEKS_API void finalize();

    namespace option { // Global option handling functions

      // Set a numerical option to `value'. `name'. Available categories and
      // options are listed in the MMA reference manual.
      CONVEKS_API void setNumber(const std::string & name,
                                 const double value);

      // Get the `value' of a numerical option.
      CONVEKS_API void getNumber(const std::string & name,
                                 double & value);

      // Print the numerical options.
      CONVEKS_API void show();

    } // namespace option

  } // namespace mma

} // namespace conveks

#endif
